PROGRAM main
    USE ISO_FORTRAN_ENV, ONLY: int32
    USE MPI
    USE HDF5
    
    IMPLICIT NONE
    
    INTEGER(int32) :: ierr
    
    ! Rank annd size of various comms
    INTEGER(int32) :: myid = -1, numprocs = -1

    ! HDF5
    INTEGER(hid_t) :: plist_id, file_id, dset_id, filespace, crp_list
    CHARACTER(len=*), PARAMETER :: filename = "fields.h5"
    INTEGER(HSIZE_T) :: dims(1)
    
    
    CALL MPI_init(ierr)
    
    CALL MPI_Comm_rank(MPI_COMM_WORLD, myid, ierr)
    CALL MPI_Comm_size(MPI_COMM_WORLD, numprocs, ierr)
    
    IF (myid == 0) THEN
        WRITE(*,*) "myid, numprocs:", myid, numprocs
    END IF

    
    ! Initialize HDF5 library
    CALL h5open_f(ierr)


    ! -------------------------------------------------------------------- !
    ! Create file
    ! -------------------------------------------------------------------- !
    ! setup file access property list with parallel I/O access.
    CALL h5pcreate_f(H5P_FILE_ACCESS_F, plist_id, ierr)
    IF (ierr < 0) CALL errr(502, 'hdf5common_open')

    CALL h5pset_fapl_mpio_f(plist_id, MPI_COMM_WORLD, MPI_INFO_NULL, ierr)
    IF (ierr /= 0) CALL errr(507, 'hdf5common_open')

    CALL h5fcreate_f(filename, H5F_ACC_TRUNC_F, file_id, ierr, &
        access_prp=plist_id)
    IF (ierr < 0) CALL errr(515, 'hdf5common_open')
    
    CALL h5pclose_f(plist_id, ierr)
    IF (ierr < 0) CALL errr(518, 'hdf5common_open')
    ! -------------------------------------------------------------------- !
    ! Leftover non-closed handlers:
    !     file_id
    ! -------------------------------------------------------------------- !
    
    
    ! -------------------------------------------------------------------- !
    ! Create dataset
    ! -------------------------------------------------------------------- !
    dims(1) = 1
    CALL h5screate_simple_f(1, dims, filespace, ierr)
    IF (ierr /= 0) CALL errr(508, 'write_runinfo')

    CALL h5pcreate_f(H5P_DATASET_CREATE_F, crp_list, ierr)
    IF (ierr /= 0) CALL errr(509, 'write_runinfo')
    
    ! BEGIN probelamtic part
    ! If these lines are commented out, the problem disappear...
    dims(1) = 1
    CALL h5pset_chunk_f(crp_list, 1, dims, ierr)
    IF (ierr /= 0) CALL errr(510, 'write_runinfo')
    ! END problematic part

    CALL h5dcreate_f(file_id, "RUNINFO", H5T_NATIVE_INTEGER, filespace, &
        dset_id, ierr, dcpl_id=crp_list)
    IF (ierr /= 0) CALL errr(511, 'write_runinfo')

    CALL h5pclose_f(crp_list, ierr)
    IF (ierr /= 0) CALL errr(512, 'write_runinfo')

    CALL h5sclose_f(filespace, ierr)
    IF (ierr /= 0) CALL errr(517, 'write_runinfo')

    CALL h5dclose_f(dset_id, ierr)
    IF (ierr /= 0) CALL errr(516, 'write_runinfo')
    
    CALL h5fclose_f(file_id, ierr)
    IF (ierr < 0) CALL errr(501, 'hdf5common_close')


    IF (myid == 0) THEN
        WRITE(*,*) "Finished!"
    END IF
    
    ! Close
    CALL h5close_f(ierr)
    CALL MPI_Finalize(ierr)

END PROGRAM main


SUBROUTINE errr(nrrr,mrrr)
    USE ISO_FORTRAN_ENV, ONLY: int32
    USE MPI, ONLY: MPI_COMM_WORLD
    
#if defined __INTEL_COMPILER
    USE IFCORE
#endif
    
    IMPLICIT NONE

    ! Variables in function call
    CHARACTER(len=*), INTENT(in) :: mrrr
    INTEGER, INTENT(in) :: nrrr
    
    ! Local variables
    INTEGER(kind=int32) :: ierr
    
    ! Print message
    write(*,*) 'ERRR NRRR=', nrrr, ' MRRR=', mrrr
    
    ! Produce backtrace to indicate from where the error is
    ! comming from   
#if defined __INTEL_COMPILER
    CALL TRACEBACKQQ(user_exit_code=-1)
#endif
#if defined __GFORTRAN__
    CALL BACKTRACE()
#endif
    
    CALL MPI_Abort(MPI_COMM_WORLD, 99, ierr)
    STOP 99
END SUBROUTINE errr
